
#include <iostream>
#include <cmath>
#include "grad.hpp"
void gradient_method::compute_gradient(double grad_w, double grad_b){
    grad_w=0.0;
    grad_b=0.0;
    for(int i=0;i<n;i++){
        grad_w=grad_w+df(omega*x[i]+b)*x[i];
        grad_w=grad_w+df(omega*x[i]+b);
        }
    grad_w=grad_w/n;
    grad_b=grad_b/n;
}

void simple_gradient::solve_gradient(int nb_iter){
    omega=0.0;
    b =0.0;
    double grad_w=0.0;
    double grad_b=0.0;
    for(int i=0;i<nb_iter;i++){
        compute_gradient(grad_w,grad_b);
        omega=omega-eta*grad_w;
        b=b-eta*grad_b;
    }    
};


void momemtum::solve_gradient(int nb_iter){
    omega=1.0;
    b =1.0;
    double grad_w=0.0;
    double grad_b=0.0;     
    compute_gradient(grad_w,grad_b);
    double v_w=grad_w, v_b=grad_b;
    for(int i=0;i<nb_iter;i++){
        compute_gradient(grad_w,grad_b);
        v_w= alpha*v_w + (1-alpha)*grad_w;
        v_b= alpha*v_b + (1-alpha)*grad_b;
        omega=omega-eta*v_w;
        b=b-eta*v_b;
    }    
};

void adagrad::solve_gradient(int nb_iter){
    omega=1.0;
    b =1.0;
    double grad_w=0.0;
    double grad_b=0.0;     
    compute_gradient(grad_w,grad_b);
    double s_w=grad_w, s_b=grad_b;
        for(int i=0;i<nb_iter;i++){
            compute_gradient(grad_w,grad_b);
            s_w = s_w + grad_w;
            s_b = s_b + grad_b;

            omega=omega-(eta/(sqrt(s_w+ eps)))*s_w;
            b=b-(eta/(sqrt(s_w+ eps)))*s_b;
        }    
};
